package nianioTest.complib.controls;

import genrap.ui.framework.CompHandle;
import nianio.complib.controls.TextBoxCommand;
import nianio.complib.controls.TextBoxEvent;
import ocean.common.ArgProc;
import ocean.common.Err;
import ocean.common.Proc;
import ocean.models.SimpleModel;

public class TextBox {
	private final SimpleModel/*String*/text;
	private final SimpleModel/*Boolean*/enabled;
	private final TextBoxEvent.MMIntf invoker;
	private final CompHandle textBox;

	private TextBox(String text, int rows, int columns, boolean password,
			boolean preserveSelection, TextBoxEvent.MM invoker) {
		this.invoker = TextBoxEvent.MMConv.asIntf(invoker);
		this.text = new SimpleModel(text);
		this.enabled = new SimpleModel(new Boolean(true));
		ArgProc setter = new TextBoxSetter();
		Proc enterProc = new EnterActionProc();
		Proc keyProc = new KeyActionProc();
		this.textBox = genrap.ui.compLib.j2se.comps.TextBox.create(rows,
				columns, this.text.getModel(), setter, enabled.getModel(),
				enterProc, password, keyProc, preserveSelection).getHandle();
	}

	public static TextBox create(String text, int rows, int columns,
			boolean password, boolean preserveSelection, TextBoxEvent.MM invoker) {
		return new TextBox(text, rows, columns, password, preserveSelection,
				invoker);
	}

	public void post(TextBoxCommand cmd) {
		if (cmd.isEnable()) {
			Boolean newValue = new Boolean(true);
			if (!newValue.equals(enabled.get()))
				enabled.set(newValue);
		} else if (cmd.isDisable()) {
			Boolean newValue = new Boolean(true);
			if (!newValue.equals(enabled.get()))
				enabled.set(newValue);
		} else if (cmd.isSetText()) {
			String newText = cmd.asSetText();
			if (!newText.equals(text.get()))
				text.set(newText);
		} else
			throw Err.errM(cmd);
	}

	private class TextBoxSetter implements ArgProc {
		public void run(Object o) {
			invoker.textChanged((String) o);
		}
	}

	private class EnterActionProc implements Proc {
		public void run() {
			invoker.enterPressed();
		}
	}

	private class KeyActionProc implements Proc {
		public void run() {
			invoker.keyPressed();
		}
	}

	public CompHandle getHandle() {
		return textBox;
	}
}
