package nianioTest.nianio.cmdsfact;

import nianio.AppLogicExtCmd;
import nianio.ExternalCommand;
import nianio.SelectFileExtCmd;
import nianio.complib.FrameCommand;
import nianio.complib.ShownFrameCommand;
import nianio.complib.controls.LabelCommand;
import nianio.complib.controls.RadioButtonGroupCommand;
import nianio.complib.controls.TextBoxCommand;
import nianio.complib.filechooser.FileChooserCommand;
import nianio.complib.messagedialog.MessageDialogCommand;
import nianio.complib.messagedialog.MessageDialogIdParams;
import nianio.genrap.GenrapPluginCommand;
import nianio.ids.FileInputId;
import nianio.ids.FrameId;
import nianio.ids.LabelId;
import nianio.ids.MessageDialogId;
import nianio.ids.RadioButtonGroupId;
import nianio.ids.TextBoxId;
import nianio.ids.WindowId;
import nianioTest.nianio.helper.UIHelper;
import ocean.common.Err;

public class ExternalCommandFact {

	public static ExternalCommand create(AppLogicExtCmd cmd) {
		if (cmd.isGenerate()) {
			return ExternalCommand.createGenrap(GenrapPluginCommand.createGenerateReport(cmd.asGenerate()));
		} else if (cmd.isShowMainFrame()) {
			return ExternalCommandHelper.createFrameCmd(FrameCommand.createCreateAndShow(UIHelper.translate(cmd.asShowMainFrame())));
		} else if (cmd.isSetOutput()) {
			RadioButtonGroupId id = RadioButtonGroupId.createOutputType();
			RadioButtonGroupCommand command = RadioButtonGroupCommand.createSelect(UIHelper.createRadioButtonId(cmd.asSetOutput()));
			return ExternalCommandHelper.createRadioButtonGroupCmd(id, command);
		} else if (cmd.isReportFile()) {
			return create(cmd.asReportFile(), FileInputId.createRaport());
		} else if (cmd.isXmlFile()) {
			return create(cmd.asXmlFile(), FileInputId.createData());
		} else if (cmd.isXsdFile()) {
			return create(cmd.asXsdFile(), FileInputId.createSchema());
		} else if (cmd.isOutputFile()) {
			return create(cmd.asOutputFile(), FileInputId.createOutput());
		} else if (cmd.isShowMessage()) {
			return ExternalCommandHelper.createMessageDialogCmd(MessageDialogCommand.createShow(MessageDialogIdParams.create(
					MessageDialogId.createGenerateFinish(cmd.asShowMessage()
							.getKey()),
					WindowId.createFrame(FrameId.createMainFrame()),
					cmd.asShowMessage().getParams())));
		} else if (cmd.isSetStatus()) {
			return ExternalCommandHelper.createLabelCmd(LabelId.createStatus(),
					LabelCommand.createSetText(cmd.asSetStatus()));
		} else if (cmd.isExit()) {
			return ExternalCommand.createExit();
		} else if (cmd.isCloseMainFrame()) {
			return ExternalCommandHelper.createFrameCmd(FrameCommand.mkFrame(
					FrameId.createMainFrame(), ShownFrameCommand.createClose()));
		} else
			throw Err.errM(cmd);
	}

	private static ExternalCommand create(SelectFileExtCmd cmd, FileInputId id) {
		if (cmd.isOpenFileChooser()) {
			return ExternalCommandHelper.createFileChooserCmd(FileChooserCommand.createCreateAndShow(UIHelper.getFileChooser(
					id, cmd.asOpenFileChooser())));
		} else if (cmd.isSetFileName()) {
			return ExternalCommandHelper.createTextBoxCmd(
					TextBoxCommand.createSetText(cmd.asSetFileName()),
					TextBoxId.createFileInput(id));
		} else
			throw Err.errM(cmd);
	}
}
