package nianioTest.nianio.helper;

import nianio.MainFrame;
import nianio.complib.CompState;
import nianio.complib.Comps;
import nianio.complib.FrameState;
import nianio.complib.Layout;
import nianio.complib.controls.Buttons;
import nianio.complib.controls.ControlState;
import nianio.complib.controls.RadioButtonState;
import nianio.complib.filechooser.FileChooserIdParams;
import nianio.complib.filechooser.FileChooserParams;
import nianio.genrap.OutputKind;
import nianio.ids.ButtonId;
import nianio.ids.FileChooserId;
import nianio.ids.FileInputId;
import nianio.ids.FrameId;
import nianio.ids.LabelId;
import nianio.ids.PanelId;
import nianio.ids.RadioButtonGroupId;
import nianio.ids.RadioButtonId;
import nianio.ids.TextBoxId;
import nianio.ids.WindowId;
import ocean.common.Err;

public class UIHelper {

	public static FrameState translate(MainFrame state) {
		Comps comps = Comps.EMPTY;
		comps = addSelectFileComps(comps, FileInputId.createRaport(),
				"Report file:", "Browse...", state.getReportFileName());
		comps = addSelectFileComps(comps, FileInputId.createData(),
				"Data file:", "Browse...", state.getXmlFileName());
		comps = addSelectFileComps(comps, FileInputId.createSchema(),
				"Schema file:", "Browse...", state.getXsdFileName());
		comps = addSelectFileComps(comps, FileInputId.createOutput(),
				"Output file:", "Browse...", state.getOutputFileName());
		comps = comps.add(
				createButton(ButtonId.createGenerate(), "Generate", true)).add(
				createLabel(LabelId.createStatus(), state.getStatus())).add(
				createRadioButtonGroup(RadioButtonGroupId.createOutputType(),
						createRadioButtonId(state.getOutputKind()),
						"Output type:", mkButtons()));

		CompState panel = CompState.mkPanel(PanelId.createContentPanel(),
				Layout.mkUniLayout(3, 5, 5, 5), comps);
		return FrameState.create(FrameId.createMainFrame(), "Nianio", true,
				panel);
	}

	private static Buttons mkButtons() {
		return Buttons.EMPTY.add(
				RadioButtonState.create(
						RadioButtonId.createPdfOutput(), "PDF",
						true)).add(
				RadioButtonState.create(
						RadioButtonId.createHtmlOutput(),
						"HTML", true)).add(
				RadioButtonState.create(
						RadioButtonId.createXlsOutput(), "XLS",
						true));
	}

	private static Comps addSelectFileComps(Comps comps, FileInputId id,
			String label, String button, String textBox) {
		return comps.add(createLabel(LabelId.createFileInput(id), label)).add(
				createTextBox(TextBoxId.createFileInput(id), 1, 30, textBox,
						false, false)).add(
				createButton(ButtonId.createFileInput(id), button, true));
	}

	public static RadioButtonId createRadioButtonId(OutputKind kind) {
		if (kind.isHtml()) {
			return RadioButtonId.createHtmlOutput();
		} else if (kind.isPdf()) {
			return RadioButtonId.createPdfOutput();
		} else if (kind.isXls()) {
			return RadioButtonId.createXlsOutput();
		} else
			throw Err.errM(kind);
	}

	private static CompState createRadioButtonGroup(RadioButtonGroupId id,
			RadioButtonId selectedId, String label, Buttons buttons) {
		return CompState.createControl(ControlState.mkRadioButtonGroup(id,
				selectedId, label, buttons));
	}

	private static CompState createButton(ButtonId id, String text,
			boolean enabled) {
		return CompState.createControl(ControlState.mkButton(id, text, enabled));
	}

	private static CompState createTextBox(TextBoxId id, int rows, int cols,
			String text, boolean password, boolean preserveSelection) {
		return CompState.createControl(ControlState.mkTextBox(id, rows, cols,
				text, password, preserveSelection));
	}

	private static CompState createLabel(LabelId id, String text) {
		return CompState.createControl(ControlState.mkLabel(id, text));
	}

	public static FileChooserIdParams getFileChooser(FileInputId id,
			FileChooserParams params) {
		if (id.isRaport()) {
			return FileChooserIdParams.create(FileChooserId.createReport(),
					WindowId.createFrame(FrameId.createMainFrame()), params);
		} else if (id.isData()) {
			return FileChooserIdParams.create(FileChooserId.createData(),
					WindowId.createFrame(FrameId.createMainFrame()), params);
		} else if (id.isSchema()) {
			return FileChooserIdParams.create(FileChooserId.createSchema(),
					WindowId.createFrame(FrameId.createMainFrame()), params);
		} else if (id.isOutput()) {
			return FileChooserIdParams.create(FileChooserId.createOutput(),
					WindowId.createFrame(FrameId.createMainFrame()), params);
		} else
			throw Err.errM(id);
	}
}
