package nianio;

import schema.runtime.datatypes.ImmObject;
import nianio.complib.CompLibIdEvent;
import nianio.genrap.GenrapPluginEvent;
import schema.runtime.err.SchemaException;
import ocean.common.Err;


/** 
 * <pre>Autogenerated class Event = variant {
 *	    complib of CompLibIdEvent  
 *	 |  genrap of GenrapPluginEvent  
 * }</pre>
 */ 
public class Event implements ImmObject {
	private static final int TYPE_HASH = 67338874;
	private int hash = 0;

	private final int variant;
	private final Object value;  


	private Event(int variant, Object value) {
		SchemaException.ensure(value != null);
		this.variant = variant;
		this.value = value;
	}


	public static Event createComplib(CompLibIdEvent value) {
		return new Event(0, value);
	}

	public static Event createGenrap(GenrapPluginEvent value) {
		return new Event(1, value);
	}



	/** @return true, when this is complib */
	public boolean isComplib() {
		return variant == 0;
	}

	/** @return variant value, when this is complib; throws Exception otherwise */
	public CompLibIdEvent asComplib() {
		SchemaException.ensure(isComplib());
		return (CompLibIdEvent)value;
	}

	/** @return true, when this is genrap */
	public boolean isGenrap() {
		return variant == 1;
	}

	/** @return variant value, when this is genrap; throws Exception otherwise */
	public GenrapPluginEvent asGenrap() {
		SchemaException.ensure(isGenrap());
		return (GenrapPluginEvent)value;
	}



	/** The only purpose of this method is to inline its content - match pattern. */ 
	public void match() {
		if (isComplib()) {
			;
		} else if (isGenrap()) {
			;
		} else 		
			throw Err.errM(this);
	}

	/** The only purpose of this method is to inline its content - 
	 * match pattern with value unpacking. 
	 */ 
	public void matchUnpack() {
		if (isComplib()) {
			asComplib();
		} else if (isGenrap()) {
			asGenrap();
		} else 		
			throw Err.errM(this);
	}	

	public String toString() {
		if (isComplib())
			return "complib" + "(" + asComplib() + ")";
		if (isGenrap())
			return "genrap" + "(" + asGenrap() + ")";
		throw SchemaException.err();
	}
	
	public boolean eq(Event obj) {
		if (obj == null)
			return false;
		if (this == obj)
			return true;
		return variant == obj.variant 
			&& (value == null ? (obj.value == null) 
			: (obj.value != null && value.equals(obj.value)));
	}

	public boolean equals(Object obj) {
		return (obj instanceof Event) 
			&& this.eq((Event)obj);
	}

	public int hashCode(){
		if (hash == 0) {
			int h = variant*(variant - 79) + 1601;
			if (value != null)
				h ^= value.hashCode();
			else
				h ^= TYPE_HASH;
			hash = h == 0 ? TYPE_HASH : h;
		}
		return hash;
	}

 	
	

	// --------------- Model Manipulator Infrastructure ----------------- //

	/**
	 *  Autogenerated Model Manipulator for Event
	 */
	public static interface MM {
		public void invoke(Event cmd);
	}
	/**
	 *  Autogenerated Model Manipulator Interface for Event
	 */
	public static interface MMIntf {
		public void complib(CompLibIdEvent value);
		public void genrap(GenrapPluginEvent value);
 	}
	/**
	 *  Autogenerated Model Manipulator Converter for Event
	 */
	public static class MMConv {
		public static MMIntf asIntf(MM mm) {
			return new MyMMIntf(mm);
		}
	
		public static MM asSimple(MMIntf mm) {
			return new MyMM(mm);
		}
	
		private static class MyMM implements MM {
	
			private final MMIntf mm;
	
			public MyMM(MMIntf mm) {
				this.mm = mm;
			}
	
			public void invoke(Event cmd) {
				 if (cmd.isComplib()) {
					mm.complib(cmd.asComplib());
				} else if (cmd.isGenrap()) {
					mm.genrap(cmd.asGenrap());
				 } else
					throw ocean.common.Err.err();				
			}
		}

		private static class MyMMIntf implements MMIntf {
			private final MM mm;
	
			public MyMMIntf(MM mm) {
				this.mm = mm;
			}
		
			public void complib(CompLibIdEvent value) {
				mm.invoke(Event.createComplib(value));
			}
		
			public void genrap(GenrapPluginEvent value) {
				mm.invoke(Event.createGenrap(value));
			}
		
		}
	}
}