package nianio.complib.controls;

import nianio.ids.RadioButtonGroupId;
import nianio.ids.ButtonId;
import schema.runtime.datatypes.ImmObject;
import nianio.ids.RadioButtonId;
import nianio.ids.TextBoxId;
import schema.runtime.err.SchemaException;
import ocean.common.Err;
import nianio.ids.LabelId;


/** 
 * <pre>Autogenerated class ControlState = variant {
 *	    button of ButtonState  
 *	 |  label of LabelState  
 *	 |  radioButton of RadioButtonState  
 *	 |  radioButtonGroup of RadioButtonGroupState  
 *	 |  textBox of TextBoxState  
 * }</pre>
 */ 
public class ControlState implements ImmObject {
	private static final int TYPE_HASH = -1469737644;
	private int hash = 0;

	private final int variant;
	private final Object value;  


	private ControlState(int variant, Object value) {
		SchemaException.ensure(value != null);
		this.variant = variant;
		this.value = value;
	}


	public static ControlState createButton(ButtonState value) {
		return new ControlState(0, value);
	}

	public static ControlState createLabel(LabelState value) {
		return new ControlState(1, value);
	}

	public static ControlState createRadioButton(RadioButtonState value) {
		return new ControlState(2, value);
	}

	public static ControlState createRadioButtonGroup(RadioButtonGroupState value) {
		return new ControlState(3, value);
	}

	public static ControlState createTextBox(TextBoxState value) {
		return new ControlState(4, value);
	}


	public static ControlState mkButton(
			ButtonId id
			, String text
			, boolean enabled
	) {
		return createButton( ButtonState.create(
							id
							, text
							, enabled
							 ));
	}
	
	public static ControlState mkLabel(
			LabelId id
			, String text
	) {
		return createLabel( LabelState.create(
							id
							, text
							 ));
	}
	
	public static ControlState mkRadioButton(
			RadioButtonId id
			, String text
			, boolean enabled
	) {
		return createRadioButton( RadioButtonState.create(
							id
							, text
							, enabled
							 ));
	}
	
	public static ControlState mkRadioButtonGroup(
			RadioButtonGroupId id
			, RadioButtonId selectedId
			, String label
			, Buttons buttons
	) {
		return createRadioButtonGroup( RadioButtonGroupState.create(
							id
							, selectedId
							, label
							, buttons
							 ));
	}
	
	public static ControlState mkTextBox(
			TextBoxId id
			, int rows
			, int cols
			, String text
			, boolean password
			, boolean preserveSelection
	) {
		return createTextBox( TextBoxState.create(
							id
							, rows
							, cols
							, text
							, password
							, preserveSelection
							 ));
	}
	

	/** @return true, when this is button */
	public boolean isButton() {
		return variant == 0;
	}

	/** @return variant value, when this is button; throws Exception otherwise */
	public ButtonState asButton() {
		SchemaException.ensure(isButton());
		return (ButtonState)value;
	}

	/** @return true, when this is label */
	public boolean isLabel() {
		return variant == 1;
	}

	/** @return variant value, when this is label; throws Exception otherwise */
	public LabelState asLabel() {
		SchemaException.ensure(isLabel());
		return (LabelState)value;
	}

	/** @return true, when this is radioButton */
	public boolean isRadioButton() {
		return variant == 2;
	}

	/** @return variant value, when this is radioButton; throws Exception otherwise */
	public RadioButtonState asRadioButton() {
		SchemaException.ensure(isRadioButton());
		return (RadioButtonState)value;
	}

	/** @return true, when this is radioButtonGroup */
	public boolean isRadioButtonGroup() {
		return variant == 3;
	}

	/** @return variant value, when this is radioButtonGroup; throws Exception otherwise */
	public RadioButtonGroupState asRadioButtonGroup() {
		SchemaException.ensure(isRadioButtonGroup());
		return (RadioButtonGroupState)value;
	}

	/** @return true, when this is textBox */
	public boolean isTextBox() {
		return variant == 4;
	}

	/** @return variant value, when this is textBox; throws Exception otherwise */
	public TextBoxState asTextBox() {
		SchemaException.ensure(isTextBox());
		return (TextBoxState)value;
	}



	/** The only purpose of this method is to inline its content - match pattern. */ 
	public void match() {
		if (isButton()) {
			;
		} else if (isLabel()) {
			;
		} else if (isRadioButton()) {
			;
		} else if (isRadioButtonGroup()) {
			;
		} else if (isTextBox()) {
			;
		} else 		
			throw Err.errM(this);
	}

	/** The only purpose of this method is to inline its content - 
	 * match pattern with value unpacking. 
	 */ 
	public void matchUnpack() {
		if (isButton()) {
			asButton();
		} else if (isLabel()) {
			asLabel();
		} else if (isRadioButton()) {
			asRadioButton();
		} else if (isRadioButtonGroup()) {
			asRadioButtonGroup();
		} else if (isTextBox()) {
			asTextBox();
		} else 		
			throw Err.errM(this);
	}	

	public String toString() {
		if (isButton())
			return "button" + "(" + asButton() + ")";
		if (isLabel())
			return "label" + "(" + asLabel() + ")";
		if (isRadioButton())
			return "radioButton" + "(" + asRadioButton() + ")";
		if (isRadioButtonGroup())
			return "radioButtonGroup" + "(" + asRadioButtonGroup() + ")";
		if (isTextBox())
			return "textBox" + "(" + asTextBox() + ")";
		throw SchemaException.err();
	}
	
	public boolean eq(ControlState obj) {
		if (obj == null)
			return false;
		if (this == obj)
			return true;
		return variant == obj.variant 
			&& (value == null ? (obj.value == null) 
			: (obj.value != null && value.equals(obj.value)));
	}

	public boolean equals(Object obj) {
		return (obj instanceof ControlState) 
			&& this.eq((ControlState)obj);
	}

	public int hashCode(){
		if (hash == 0) {
			int h = variant*(variant - 79) + 1601;
			if (value != null)
				h ^= value.hashCode();
			else
				h ^= TYPE_HASH;
			hash = h == 0 ? TYPE_HASH : h;
		}
		return hash;
	}

 	
	
}