#!/usr/bin/perl

my $dir_lang = "../../../ui/genrap/ui/lang/lang";
my @java_path = ("out", "genrap", "ui", "lang");
my $dir_ui = "../../../../../ui/genrap/ui";
my $base_file = "pl.lang";
my $dictionary_file = "Dictionary.java";
my $lang_file = "Lang.java";
my $package = "genrap.ui.lang";

my %dict;
my %lang;
$lang_files = &read_lang_dir;

my @names;

sub my_die {
	my ($msg) = @_;
	print $msg;
	exit (1);
}

$_ = $base_file;
$name = $1 if /(.+).lang/i or my_die("Wrong file name $base_file.");
push(@names, $name);

@lang_files = @$lang_files;

for ($i=0; $i <= $#lang_files; $i++) {
	$lang_f = $lang_files[$i];
	if ($lang_f ne $base_file) {
		$_ = $lang_f;
		$name = $1 if /(.+).lang/i or my_die("Wrong file name $lang_f.");
		push(@names, $name);
	}
}
#################################################################################################

my @imports = (
	"ocean.models.Model"
	);


sub read_base_lang {
	my ($lang_file) = @_;
	my %lang_dict;
	my $i = 0;

	chdir $dir_lang;

	open LANG_FILE, $lang_file or my_die("Cannot open $lang_file for read :$!");

	while (<LANG_FILE>) {
		if (!/^[\#]+/i  && !/^[\s]*$/i) {
			$lang_dict{$1} = $2 if /([A-Z0-9_]+)\s+([^\n\r]*)/i 
					or my_die("Wrong file $lang_file format in line $. : $_.");
		}
	}

	close LANG_FILE;
	return %lang_dict;
}

sub read_lang {
	my ($lang_file, %base_lang) = @_;

	my %lang_dict;
	my $i = 0;

	chdir $dir_lang;

	open LANG_FILE, $lang_file or my_die("Cannot open $lang_file for read :$!");

	while (<LANG_FILE>) {
		if (!/^[\#]+/i  && !/^[\s]*$/i) {
			$lang_dict{$1} = $2 if /([A-Z0-9_]+)\s+([^\n\r]*)/i 
					or my_die("Wrong file $lang_file format in line $. : $_.");
			if (!defined $base_lang{$1}) {
				my_die("In file $lang_file, line $.: the key $1 is missing in the base file.");
			}

		}
	}

	close LANG_FILE;
	return %lang_dict;
}

sub read_lang_dir {
	my @files;
	chdir $dir_lang or my_die("Cannot chdir to $dir_lang:$!\n") if $dir_lang;

	while (<*.lang>) {
		push(@files, "$_");
	}
	return \@files;
}


sub print_model {
	my ($stream, $word, $argc) = @_;

	$_ = ucfirst (lc $word);
	/([^_]+)/i;
	my $model_name = $1;
	my @parts = /_([a-z0-9]+)/ig;

	foreach $part (@parts) {
		$part = ucfirst($part);
        	$model_name = "$model_name$part";
	}
	
	if ($argc == 0) {
		print $stream "	public Model get$model_name() {\n";
		print $stream "		return new DictionaryM($word, langModel);\n";
		print $stream "	}\n";
	} elsif ($argc == 1) {
		print $stream "	public Model get$model_name(Model model1) {\n";
		print $stream "		return new DictionaryM($word, langModel, model1);\n";
		print $stream "	}\n";
	} elsif ($argc > 1) {
		print $stream "	public Model get$model_name(Model[] models) {\n";
		print $stream "		return new DictionaryM($word, langModel, models);\n";
		print $stream "	}\n";
	}
}

sub print_array {
	my ($stream, $word, $tab) = @_;
	my $first = 1;
	my $argc = 0;

	print $stream "	private final String[] $word = {";

	foreach my $w (@$tab) {
		if ($first) {
			$first = 0;
	
			$_ = $w;
			my @argv = /(%)/ig;
			$argc = $#argv + 1;
			
		} else {
			print $stream ", ";
		}
		print $stream "\"$w\"";
	}
	print $stream "};\n";
	return $argc;
}

sub print_imports {
	my ($stream) = @_;

	foreach my $import (@imports) {
		print $stream "import $import;\n";
	}
	print $stream "\n";
}

sub print_header {
	my ($stream) = @_;

	print $stream "package $package;\n\n";
	&print_imports($stream);

	print $stream "public class Dictionary {\n\n";
	print $stream "	private Model langModel;\n\n";

}

sub print_constructor {
	my ($stream) = @_;

	print $stream "\n";
	print $stream "	public Dictionary(Model langM) {\n";
	print $stream "		this.langModel = langM;\n";
	print $stream "	}\n";

}

sub print_footer {
	my ($stream) = @_;

	print $stream "}\n";

}

sub print_dictionary {

	%base_lang = &read_base_lang($base_file);
	foreach (keys %base_lang) {
		my @word_tab = ($base_lang{$_});
		$dict{$_} = \@word_tab;
	}

	foreach $lang_file (@lang_files) { 
		if ($lang_file ne $base_file) {
	
			%lang = &read_lang($lang_file, %base_lang);
		
			foreach $word (keys %dict) {
				$word_tab=$dict{$word};
				if (!defined $lang{$word}) {
					my_die("Wrong file $lang_file format, $word key missing.");
				}
				push(@$word_tab, $lang{$word});
			}		
		}
	}

	chdir $dir_ui or my_die("Invalid java directory: $dir_ui.");
	
	for ($i=0; $i <= $#java_path; $i++) {
		if (!(chdir $java_path[$i])) {
			mkdir $java_path[$i];
			chdir $java_path[$i] or my_die("Invalid java directory: $java_path[$i].");
		}
	}
	open $OUT, ">$dictionary_file" or my_die("Cannot open $dictionary_file for writing.");

	my $i = 0;

	&print_header($OUT);

	foreach $word (keys %dict) {
		$word_tab=$dict{$word};
		$argc = &print_array($OUT, $word, $word_tab);
		&print_model($OUT, $word, $argc);
	}
	&print_constructor($OUT);
	&print_footer($OUT);

	close $OUT;
}

##################################################################################################

sub print_lang_header {
	my ($stream) = @_;

	print $stream "package $package;\n\n";

	print $stream "import ocean.common.Err;\n\n";

	print $stream "public class Lang {\n\n";
	
	print $stream "	private int variant;\n";

}

sub print_lang_names {
	my ($stream, $nms) = @_;
	my $first = 1;
	my @names = @$nms;

	print $stream "	private final static String[] names = {";

	for ($i=0; $i <= $#names; $i++) {
		if ($first) {
			$first = 0;
		} else {
			print $stream ", ";
		}
		chop $w;

		print $stream "\"$names[$i]\"";
	}
	print $stream "};\n";
}

sub print_lang_static_subroutines {
	my ($stream) = @_;

	print $stream "	public static Lang[] getLangs() {\n";
	print $stream "		int n = names.length;\n";
	print $stream "		Lang[] langs = new Lang[n];\n";
	print $stream "		for (int i=0; i<n; i++) {\n";
	print $stream "			langs[i] = new Lang(i);\n";
	print $stream "		}\n";
	print $stream "		return langs;\n";
	print $stream "	}\n\n";
		
}

sub print_lang_constructors {
	my ($stream, $nms) = @_;
	my @names = @$nms;

	for ($i=0; $i <= $#names; $i++) {
		$name = uc $names[$i];
		print $stream "	private static final Lang $name = new Lang($i);\n";
	}
	print $stream "\n";
	
	print $stream "	private Lang(int variant) {\n";
	print $stream "		this.variant = variant;\n";
	print $stream "	}\n\n";

	print $stream "	public Lang(String name) {\n";
	for ($i=0; $i <= $#names; $i++) {
		$optName = ucfirst $names[$i];
		print $stream "		if (name.equals(\"$names[$i]\")) {\n";
		print $stream "			this.variant = $i;\n";
		print $stream "		} else \n";
	}
	print $stream "			this.variant = 0;\n";
	print $stream "	}\n\n";

	for ($i=0; $i <= $#names; $i++) {
		$name = uc $names[$i];
		$fucName = ucfirst $names[$i];
		print $stream "	public static Lang create$fucName() {\n";
		print $stream "		return $name;\n";
		print $stream "	}\n";
	}
	print $stream "\n";

}

sub print_lang_subroutines {
	my ($stream, $nms) = @_;
	my @names = @$nms;

	print $stream "	public String getName() {\n";
	print $stream "		String name = names[variant];\n";
	print $stream "		Err.errN(name);\n";
	print $stream "		return name;\n";
	print $stream "	}\n";
	print $stream "	public int getIndex() {\n";
	print $stream "		return variant;\n";
	print $stream "	}\n\n";

	for ($i=0; $i <= $#names; $i++) {
		$name = uc $names[$i];
		$fucName = ucfirst $names[$i];
		print $stream "	public boolean is$fucName() {\n";
		print $stream "		return variant == $i;\n";
		print $stream "	}\n";
	}
	print $stream "\n";
	
	print $stream "	public boolean is(String name) {\n";
	print $stream "		return name.equals(names[this.variant]);\n";
	print $stream "	}\n\n";
	
}

sub print_lang_footer {
	my ($stream) = @_;

	print $stream "}\n";

}

sub print_lang {

	open $OUT, ">$lang_file" or my_die("Cannot open $lang_file for writing.");

	&print_lang_header($OUT);

	&print_lang_names($OUT, \@names);
	&print_lang_static_subroutines($OUT);
	&print_lang_constructors($OUT, \@names);
	&print_lang_subroutines($OUT, \@names);

	&print_lang_footer($OUT);

	close $OUT;

}
###########################################################################################

&print_dictionary;
&print_lang;
